package com.cnbm.processInspection.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.cnbm.basic.entity.Machine;
import com.cnbm.basic.entity.Product;
import com.cnbm.basic.entity.Shift;
import com.cnbm.basic.entity.WorkingProcedure;
import com.cnbm.basic.service.impl.MachineServiceImpl;
import com.cnbm.basic.service.impl.ProductServiceImpl;
import com.cnbm.basic.service.impl.ShiftServiceImpl;
import com.cnbm.basic.service.impl.WorkingProcedureServiceImpl;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.influx.config.InfluxClient;
import com.cnbm.influx.param.QueryDataParam;
import com.cnbm.influx.param.Range;
import com.cnbm.influx.template.Event;
import com.cnbm.processInspection.dto.InspectionSampleDTO;
import com.cnbm.processInspection.dto.InspectionSheetDTO;
import com.cnbm.processInspection.entity.InspectionSheet;
import com.cnbm.processInspection.mapper.InspectionSheetMapper;
import com.cnbm.processInspection.service.IInspectionSheetService;
import com.influxdb.query.FluxTable;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;

/**
 * 检验单 表
 *
 * @author why
 * @since 2022-08-17
 */
@Service
public class InspectionSheetServiceImpl extends CrudServiceImpl<InspectionSheetMapper, InspectionSheet, InspectionSheetDTO> implements IInspectionSheetService {

    @Autowired
    private ProductServiceImpl productService;

    @Autowired
    private WorkingProcedureServiceImpl workingProcedureService;

    @Autowired
    private MachineServiceImpl machineService;

    @Autowired
    private ShiftServiceImpl shiftService;
    
    @Override
    public QueryWrapper<InspectionSheet> getWrapper(Map<String, Object> params){
        Long id = (Long)params.get("id");
        String inspectionSite = (String)params.get("inspectionSite");
        String orderNumber = (String)params.get("orderNumber");
        String batchNumber = (String)params.get("batchNumber");

        QueryWrapper<InspectionSheet> wrapper = new QueryWrapper<>();
        wrapper.like(ObjectUtils.isNotNull(id), "id", id);
        wrapper.like(StringUtils.isNotBlank(inspectionSite), "inspection_stage", inspectionSite);
        wrapper.like(StringUtils.isNotBlank(orderNumber), "order_number", orderNumber);
        wrapper.like(StringUtils.isNotBlank(batchNumber), "batch_number", batchNumber);
        wrapper.eq(ObjectUtils.isNotNull(params.get("productId")), "product_id", params.get("productId"));
        wrapper.eq(ObjectUtils.isNotNull(params.get("workingProcedureId")), "working_procedure_id", params.get("workingProcedureId"));
        wrapper.eq(ObjectUtils.isNotNull(params.get("machineId")), "machine_id", params.get("machineId"));
        return wrapper;
    }
    
    @Override
    public PageData<InspectionSheetDTO> page (Map<String, Object> params){
        IPage<InspectionSheet> page = baseDao.selectPage(
                getPage(params, "id", true),
                getWrapper(params)
        );
        PageData<InspectionSheetDTO> inspectionSheetDTOPageData = getPageData(page, InspectionSheetDTO.class);

        if(!inspectionSheetDTOPageData.getList().isEmpty()){
            for(InspectionSheetDTO dto:inspectionSheetDTOPageData.getList()){
                Product product = productService.selectById(dto.getProductId());
                WorkingProcedure workingProcedure = workingProcedureService.selectById(dto.getWorkingProcedureId());
                Machine machine = machineService.selectById(dto.getMachineId());
                Shift shift = shiftService.selectById(dto.getShiftId());

                String productName = null;
                String workingProcedureName = null;
                String machineName = null;
                String shiftName = null;

                if (product!=null){
                    productName = product.getName();
                }
                if (workingProcedure!=null){
                    workingProcedureName = workingProcedure.getName();
                }
                if (machine!=null){
                    machineName = machine.getName();
                }
                if (shift!=null){
                    shiftName = shift.getName();
                }

                dto.setProductName(productName);
                dto.setWorkingProcedureName(workingProcedureName);
                dto.setMachineName(machineName);
                dto.setShiftName(shiftName);
            }
        }
        return inspectionSheetDTOPageData;
    }

    @Override
    public InspectionSheetDTO get(Long id) {
        InspectionSheet entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, InspectionSheetDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(InspectionSheetDTO dto) {
        InspectionSheet entity = ConvertUtils.sourceToTarget(dto, InspectionSheet.class);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(InspectionSheetDTO dto) {
        InspectionSheet entity = ConvertUtils.sourceToTarget(dto, InspectionSheet.class);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Autowired
    private InspectionSheetMapper mapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<InspectionSheetDTO> list(Map<String, Object> params) {
        List<InspectionSheetDTO> list = mapper.list(params);
        return list;
    }

    @Override
    public List<FluxTable> getFluxParamList(Map<String, Object> params){
        QueryDataParam queryDataParam = new QueryDataParam();
        String workingProcedureName = (String)params.get("workingProcedureName");
        queryDataParam.setMeasurement(workingProcedureName);
        queryDataParam.setBucket("qgs-bucket");

        Instant startTime = (Instant) params.get("startTime");
        Instant endTime = (Instant) params.get("endTime");
        Range range = new Range(startTime,endTime);
        queryDataParam.setRange(range);

        List<FluxTable> list = InfluxClient.Client.query(queryDataParam);
        return list;
    }

    @Override
    public void saveFluxParamList(@RequestBody InspectionSampleDTO[] lists){
        for(InspectionSampleDTO dto:lists){
            String equipmentName = dto.getWorkingProcedureName();
            LocalDateTime eventTimestamp = LocalDateTime.now();
            Instant eventTime = eventTimestamp.toInstant(ZoneOffset.UTC);
            String inspectionSheetId =dto.getInspectionSheetId();
//            Event event = new Event(eventTime,inspectionSheetId,"ppExecName",ppExecName);
//            InfluxClient.Client.insert(event,equipmentName);
        }
    }

    private Event newEvent(Instant time,String inspectionSheetId,String argName,Double argValue){
        Event event = new Event();
        event.setInspectionSheetId(inspectionSheetId);
        event.setTime(time);
        event.setArgName(argName);
        if(!Objects.equals(argValue, "") && argValue != null ){
            event.setArgValue(argValue.toString());
        }
        return event;
    }
}